[CUDA] 使用thrust::sort()函数排序

一、简介

本文介绍了如何使用thrust::sort()函数对device_vector<int>容器,或者int*指针表示的内存(显存)处的数据进行排序。

二、示例代码

1. 排序 device_vector<int>中的数据

main.cu文件内容:

cpp 复制代码
#include <iostream>
#include <cuda_runtime.h>
#include <thrust/device_vector.h>
#include <thrust/device_ptr.h>
#include <thrust/host_vector.h>
#include <thrust/sort.h>
#include <algorithm>
#include <random>
#include <chrono>

using namespace std;
int main()
{

    int min = 0, max = 100;
    random_device seed;                           // 硬件生成随机数种子
    ranlux48 engine(seed());                      // 利用种子生成随机数引擎
    uniform_int_distribution<> distrib(min, max); // 设置随机数范围,并为均匀分布

    int n = 10;
    thrust::host_vector<int> data_host(n);
    thrust::device_vector<int> data_device(n);

    /* 在host_vector上生成随机数 */
    for (int i = 0; i < n; i++)
    {
        data_host[i] = distrib(engine);
    }
    std::cout << "Before sorting:\n";
    for (int i = 0; i < n; i++)
    {
        std::cout << data_host[i] << ",";
    }
    std::cout << "\n";

    /* 在device端进行排序 */

    data_device = data_host;                              // 将host端的数据拷贝到device端
    thrust::sort(data_device.begin(), data_device.end()); // 进行排序
    data_host = data_device;                              // 将排序完成后的device端的数据拷贝回host端

    std::cout << "After sorting:\n";
    for (int i = 0; i < n; i++)
    {
        std::cout << data_host[i] << ",";
    }
    std::cout << "\n";

    return 0;
}

CMakeLists.txt文件内容:

cpp 复制代码
cmake_minimum_required(VERSION 3.8 FATAL_ERROR)
project(HelloWolrd LANGUAGES CXX CUDA)
set(CMAKE_CXX_STANDARD 14)
set(CMAKE_CXX_STANDARD_REQUIRED ON)
# CUDA设置
find_package(CUDA REQUIRED)
# 指定生成目标
cuda_add_executable(HelloWorld main.cu)

程序输出结果:

cpp 复制代码
Before sorting:
85,73,8,0,77,68,93,31,85,76,
After sorting:
0,8,31,68,73,76,77,85,85,93,

2. 排序 int*指针中的数据

main.cu文件内容:

cpp 复制代码
#include <iostream>
#include <cuda_runtime.h>
#include <thrust/device_vector.h>
#include <thrust/device_ptr.h>
#include <thrust/host_vector.h>
#include <thrust/sort.h>
#include <algorithm>
#include <random>
#include <chrono>

using namespace std;
int main()
{

    int min = 0, max = 100;
    random_device seed;                           // 硬件生成随机数种子
    ranlux48 engine(seed());                      // 利用种子生成随机数引擎
    uniform_int_distribution<> distrib(min, max); // 设置随机数范围,并为均匀分布

    int n = 10;
    int *data_host = new int[n];
    /* 生成随机数 */
    for (int i = 0; i < n; i++)
    {
        data_host[i] = distrib(engine);
    }
    std::cout << "Befor sorting:\n";
     for (int i = 0; i < n; i++)
    {
        std::cout << data_host[i] << ",";
    }
    std::cout << "\n";    

    /*数据拷贝到device端*/
    int *data_device = nullptr;
    cudaMalloc((void **)&data_device, n * sizeof(int));
    cudaMemcpy(data_device, data_host, n * sizeof(int), cudaMemcpyHostToDevice);

    /*使用thrust::sort()排序*/
    // thrust::sort()只支持迭代器,因此需要将int*指针转为device_ptr<int>容器类型
    thrust::device_ptr<int> t_ptr(data_device);
    thrust::sort(t_ptr, t_ptr + n);

    /*数据拷贝回host端*/
    cudaMemcpy(data_host, data_device, n * sizeof(int), cudaMemcpyDeviceToHost);
    
    std::cout << "After sorting:\n";
     for (int i = 0; i < n; i++)
    {
        std::cout << data_host[i] << ",";
    }
    std::cout << "\n";
    /*释放内存*/
    delete[] data_host;
    cudaFree(data_device);

    return 0;
}

CMakeLists.txt文件内容同上。

程序输出结果:

cpp 复制代码
Befor sorting:
5,82,71,3,90,69,8,0,23,81,
After sorting:
0,3,5,8,23,69,71,81,82,90,

三、自定义排序

更多自定义排序请参考thrust官方代码仓库:NVIDIA/thrust

相关推荐
颜x小2 小时前
[C#] C++与c#语法对比
开发语言·c++·c#
汉克老师3 小时前
GESP2026年3月认证C++八级( 第三部分编程题(2、子图最短路)精讲
c++·最短路·floyd·gesp8级
abcd_zjq4 小时前
海康线扫相机使用
c++·海康
十五年专注C++开发4 小时前
C++设计一个函数来求容器元素绝对值之和的平均值
c++·模板·accumulate
SilentSlot4 小时前
【C/C++】手写 DPDK 协议栈(十一):基于 PPS、SYN 比例和源 IP 熵的 DDoS 检测
c语言·c++·tcp/ip
王老师青少年编程5 小时前
csp信奥赛C++高频考点专项训练:【排序算法】案例9:成绩排序
c++·排序算法·csp·高频考点·信奥赛·成绩排序
程序喵大人6 小时前
【C++进阶】STL容器与迭代器 - 05 map 和 set 为什么按键保持有序
开发语言·c++·容器·迭代器·stl
ShineWinsu6 小时前
对于Linux:HTTP中cookie、session的解析
linux·网络·c++·网络协议·http·cookie·session
库克克6 小时前
【C++】 unordered_map 与unordered_set
开发语言·c++
李迟8 小时前
一种轻量级C++ CSV文件读写库的实现方案
开发语言·c++